
#ifndef CONTR_RELAY_H_QWERTY
#define CONTR_RELAY_H_QWERTY

void relaytx_init(void);
void relaytx_on(void);
void relaytx_off(void);
void relaytx_onoff(void);

void relay10m_init(void);
void relay10m_on(void);
void relay10m_off(void);

void relay20m_init(void);
void relay20m_on(void);
void relay20m_off(void);

void relay40m_init(void);
void relay40m_on(void);
void relay40m_off(void);

void relay80m_init(void);
void relay80m_on(void);
void relay80m_off(void);

void buzzer_init(void);
void buzzer_on(void);
void buzzer_off(void);
void buzzer_onoff(void);

void fan_init(void);
void fan_on(void);
void fan_off(void);
void fan_onoff(void);
bool fan_is_on(void);

void atten_init(void);
void atten_on(void);
void atten_off(void);

#endif
